/***********************************************************************
 * Universal Asznchronus Receiver Transmitter
 ***********************************************************************
 * Function:  Provice UART functions and methods.
 ***********************************************************************
 * Processor:  ATmega168
 * Clock:    F_CPU
 * Language:  ANSI-C
 * Compiler:  avr-gcc/4.1.3, avr-libc
 * Date:    $LastChangedDate$
 * Author:    $LastChangedBy$
 * ID:      $Id$
 ***********************************************************************
 * License:    Creative Commons Attribution-Share Alike 3.0 Germany
 *        <http://creativecommons.org/licenses/bz-sa/3.0/>
 * Copyright:  2009, Soeren Kornetzki <soeren.kornetzki@dev5-media.de>
 * Attribution:  You are not allowed to remove any license, copyright or
 *        attribution notice. In addition you have to inform your
 *        users about the using of this sourc code ( bundle ) in
 *        the following form ( you are allowed to translate ):
 *            This software is using #SourceCodeBundleName#
 *            library created by Soeren Kornetzki ( 
 *            http://dev5-media.de ) licensed under the creative
 *            commons attribution-share alike 3.0 german license.
 *        Replace #SourceCodeBundleName# with the source code
 *        bundle name.
 **********************************************************************/
 #define F_CPU 20000000UL
/* multiple include prevention  */
#ifndef  _UART_C_
#define  _UART_C_  1

/* define UART symbols per second, if not defined  */
#ifndef  UART_BAUD
#define  UART_BAUD    9600UL
#endif

/* include librarys  */
#include <avr/io.h>
#include <avr/interrupt.h>

/* global variables  */

/* prototypes  */
void uart_init( void );
char uart_getc( void );
void uart_putc( char b );
void uart_puts( char* bs );

/***********************************************************************
 * uart_init
 * initialize the UART
 ***********************************************************************
 * Baud:  UART_BAUD
 * Parity:  Disabled
 * Stop:  1-bit
 * Char:  8-bit
 **********************************************************************/
void uart_init( void )
{
  /* USART receiver, transmitter enable and interrupt handling  */
#ifdef UART_RXD_INT
#ifdef UART_TXD_INT
  /* RXD and TXD interrupt  */
  UCSR0B = ( _BV( RXCIE0 ) | _BV( TXCIE0 ) | _BV( RXEN0 ) | _BV( TXEN0 ) );
#else
  /* RXD interrupt  */
  UCSR0B = ( _BV( RXCIE0 ) | _BV( RXEN0 ) | _BV( TXEN0 ) );
#endif
#else
#ifdef UART_TXD_INT
  /* TXD interrupt  */
  UCSR0B = ( _BV( TXCIE0 ) | _BV( RXEN0 ) | _BV( TXEN0 ) );
#else
  /* no interrupt  */
  UCSR0B = ( _BV( RXEN0 ) | _BV( TXEN0 ) );
#endif
#endif

  /* USART mode: UART with no parity, 1-bit stop and 8-bit chars  */
  UCSR0C = ( _BV( UMSEL00 ) | _BV( UCSZ00 ) | _BV( UCSZ01 ) );

  /* calculate UART baud rate  */
  UBRR0 = ( ( ( F_CPU / UART_BAUD ) / 16 ) - 1 );
}

/***********************************************************************
 * uart_getc
 * returns last received byte by UART. If no byte received, wait for it
 ***********************************************************************
 * @return  char  last received byte from UART
 **********************************************************************/
char uart_getc( void )
{
  char data = 0;
  while( !( UCSR0A & _BV( RXC0 ) ) ); /* waits until RX is completed  */
  data = UDR0;
  return ( char )data;
}

/***********************************************************************
 * uart_putc
 * transmit byte via UART
 ***********************************************************************
 * @param  char  b  byte to transmit via UART
 **********************************************************************/
void uart_putc( char b )
{
  while( !( UCSR0A & _BV( UDRE0 ) ) ); /* waits until preview TX is completed  */
  UDR0 = ( uint8_t )b;
}

/***********************************************************************
 * uart_puts
 * transmit string of bytes via UART
 ***********************************************************************
 * @param  char*  bs  string of bytes to transmit via UART
 **********************************************************************/
void uart_puts( char* bs )
{
  uint8_t i;
  for( i = 0; bs[i] != 0; i++ )
    uart_putc( bs[i] );
}

#endif /* multiple include prevention  */